﻿using System;
using System.Windows.Forms;
using Microsoft.Win32;
using System.Drawing;
using System.Drawing.Printing;
using System.ComponentModel;

namespace EasyZVT
{
    public partial class FormMain : Form
    {
        // Erstellt durch Jan Groetzner, Groetzner Unternehmensberatung, Seevetal
        // Version: 17.08.2018 
        // Kontakt: easyzvt@groetzner.de

        // Dieser Quellcode dient als Beispielimplementierung zur Einbindung von EasyZVT in die eigene Kassenanwendung.
        // Der Code und auch Teile davon dürfen frei kopiert und in der Kassenanwendung verwendet werden 

        BackgroundWorker EasyZVT_Worker = new BackgroundWorker();
        int Funktion;
        int Betrag = 0;
        PrintDocument PDocument;

 
        public FormMain()
        {
            InitializeComponent();
        
            // Anwendungsstart
            // Worker-Thread wird initialisiert 
            // Vorherige Werte werden aus der Registry gelesen und in die Oberfläche eingetragen
            // Die Werte in der Oberfläche sollten in der echten Anwendung in der Stammdatentabelle der Kasse gespeichert werden und nicht in der Registry. Geht aber natürlich auch hier.

            try
            {
                EasyZVT_Worker.DoWork += new DoWorkEventHandler(EasyZVT_Async);
                EasyZVT_Worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(EasyZVT_OnCompleted);

                RegistryKey ZVTKey = Registry.CurrentUser.OpenSubKey(@"Software\GUB\ZVT", true);
                TextBox_KasseNr.Text = Convert.ToString(ZVTKey.GetValue("KasseNr", "1"));
                ComboBox_COM.Text = Convert.ToString(ZVTKey.GetValue("COM", "LAN"));
                TextBox_IP.Text = Convert.ToString(ZVTKey.GetValue("IP", "127.0.0.1"));
                TextBox_Port.Text = Convert.ToString(ZVTKey.GetValue("Port", 5577));
                ComboBox_ComSpeed.Text = Convert.ToString(ZVTKey.GetValue("ComSpeed", 9600));
                ComboBox_ComStop.Text = Convert.ToString(ZVTKey.GetValue("ComStop", 1));
                TextBox_Lizenz.Text = Convert.ToString(ZVTKey.GetValue("Lizenz", ""));

                if (Convert.ToInt32(ZVTKey.GetValue("Kassedruck", 0)) == 1)
                {
                    CheckBox_Kassedruck.Checked = true;
                }
                else
                {
                    CheckBox_Kassedruck.Checked = false;
                }

                ComboBox_Typ.SelectedIndex = ComboBox_Typ.FindString(Convert.ToString(ZVTKey.GetValue("Typ", 0)));
                ZVTKey.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Fehler beim Einlesen der Standardwerte: " + ex.Message);
            }
        }

        void SetRegistryParameter()
        {
            // Funktion holt Werte direkt aus der Oberfläche (TextBox_...) und schreibt diese in die Registry als Eingabeparameter für EasyZVT. 
            // Im wahren Leben stehen diese Werte in Datenbankeinträgen der Stammdatenverwaltung in der Hauptanwendung

            RegistryKey ZVTKey = Registry.CurrentUser.OpenSubKey(@"Software\GUB\ZVT", true);
            string Typ;

            try
            {
                ZVTKey.SetValue("Funktion", Funktion, RegistryValueKind.DWord);
                Typ = ComboBox_Typ.Text.Split(':')[0];
                ZVTKey.SetValue("Typ", Typ, RegistryValueKind.DWord);
                ZVTKey.SetValue("KasseNr", TextBox_KasseNr.Text, RegistryValueKind.String);
                ZVTKey.SetValue("COM", ComboBox_COM.Text, RegistryValueKind.String);
                ZVTKey.SetValue("IP", TextBox_IP.Text, RegistryValueKind.String);
                ZVTKey.SetValue("Port", (TextBox_Port.Text == "") ? 0 : Convert.ToInt32(TextBox_Port.Text), RegistryValueKind.DWord);
                ZVTKey.SetValue("ComSpeed", Convert.ToInt32(ComboBox_ComSpeed.Text), RegistryValueKind.DWord);
                ZVTKey.SetValue("ComStop", Convert.ToInt32(ComboBox_ComStop.Text), RegistryValueKind.DWord);
                ZVTKey.SetValue("Lizenz", TextBox_Lizenz.Text, RegistryValueKind.String);

                if (Funktion == 0 | Funktion == 4)
                {
                    Betrag = Convert.ToInt32("0" + TextBox_Betrag.Text);
                    ZVTKey.SetValue("Betrag", Betrag, RegistryValueKind.DWord);
                }

                if (Funktion == 3)
                {
                    ZVTKey.SetValue("StornoBetrag", (TextBox_Betrag.Text == "") ? 0 : Convert.ToInt32(TextBox_Betrag.Text), RegistryValueKind.DWord);
                    ZVTKey.SetValue("StornoBelegNr", (TextBox_BelegNr.Text == "") ? 0 : Convert.ToInt32(TextBox_BelegNr.Text), RegistryValueKind.DWord);
                }

                if (CheckBox_Kassedruck.Checked)
                {
                    ZVTKey.SetValue("Kassedruck", 1, RegistryValueKind.DWord);
                }
                else
                {
                    ZVTKey.SetValue("Kassedruck", 0, RegistryValueKind.DWord);
                }

                Button_Autorisierung.Enabled = false;
                Button_Diagnose.Enabled = false;
                Button_Tagesabschluss.Enabled = false;
                Button_Storno.Enabled = false;
                Button_Gutschrift.Enabled = false;
                Button_Exit.Enabled = false;

                TextBox_ErgebnisText.Text = "EasyZVT-Zahlung aktiv";
                TextBox_Aktiv.Text = "warte auf Aktiv=0";
            }
            catch (Exception ex)
            {
                MessageBox.Show("Das Schreiben der Parameter ist fehlgeschlagen: " + ex.Message);
                return;
            }
            ZVTKey.Close();
        }

        void Autorisierung()
        {
            Funktion = 0;
            SetRegistryParameter();
            EasyZVT_Worker.RunWorkerAsync();
        }

        void Button_Autorisierung_Click(object sender, EventArgs e)
        {
            // Zahlung per Klick auslösen
            Autorisierung();
        }

        void TextBox_Betrag_KeyDown(object sender, KeyEventArgs e)
        {
            // Damit die Zahlung auch per Enter-Taste ausgelöst werden kann
            if (e.KeyCode == Keys.Enter)
            {
                e.Handled = true;
                Autorisierung();
            }
        }

        void EasyZVT_Async(object sender, DoWorkEventArgs e)
        {
            // Programmstart von ZVTStart.exe und warten auf Programmende
            RegistryKey ZVTKey = Registry.CurrentUser.OpenSubKey(@"Software\GUB\ZVT", true);

            try
            {
                string ZVTStart = (string)ZVTKey.GetValue("Start");
                System.Diagnostics.Process Proc;

                if (ZVTStart.Length > 1)
                {
                    ZVTKey.SetValue("Ergebnis", 1000, RegistryValueKind.DWord); // Wenn das Ergebnis 1000 bleibt konnte EasyZVT nicht gestartet werden
                    ZVTKey.SetValue("ErgebnisText", "EasyZVT konnte nicht gestartet werden", RegistryValueKind.String);

                    Proc = System.Diagnostics.Process.Start(ZVTStart);
                    System.Threading.Thread.Sleep(3000);    // Warte drei Sekunden bis Prozess ZVTStart geladen ist, dann erst anfangen Flag Aktiv abzufragen
                    while (Convert.ToInt32( ZVTKey.GetValue("Aktiv")) == 1)
                    {
                        System.Threading.Thread.Sleep(500); // Warten...
                        /*if (TextBox_Aktiv.BackColor == Color.Gray)
                        {
                            TextBox_Aktiv.BackColor = Color.Red;
                        }
                        else
                        {
                            TextBox_Aktiv.BackColor = Color.Gray;
                        }*/
                    }
                    //TextBox_Aktiv.BackColor = Color.Gray;
                }
                else
                    MessageBox.Show("EasyZVT ist nicht installiert. Bitte im Internet Explorer über www.easyzvt.de/publish installieren");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Der Programmstart ist fehlgeschlagen: " + ex.Message);
                return;
            }
            ZVTKey.Close();
        }

        void EasyZVT_OnCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            // Diese Funktion wird automatisch aufgerufen wenn der Worker-Thread seine Arbeit beendet
            // und EasyZVT seine Ausgabewerte in die Registry zurückgeschrieben hat
            RegistryKey ZVTKey = Registry.CurrentUser.OpenSubKey(@"Software\GUB\ZVT", true);

            TextBox_Aktiv.Text = "";
            TextBox_Ergebnis.Text = Convert.ToString(ZVTKey.GetValue("Ergebnis"));
            TextBox_ErgebnisText.Text = Convert.ToString(ZVTKey.GetValue("ErgebnisText"));
            TextBox_Drucktext.Text = Convert.ToString(ZVTKey.GetValue("Drucktext")).Replace("\n","\r\n");
            TextBox_Drucktext2.Text = Convert.ToString(ZVTKey.GetValue("Drucktext2")).Replace("\n","\r\n");

            TextBox_BelegNr.Text = Convert.ToString(ZVTKey.GetValue("BelegNr"));
            TextBox_Betrag.Text = "";
            TextBox_Betrag.Focus();
            Button_Autorisierung.Enabled = true;
            Button_Diagnose.Enabled = true;
            Button_Tagesabschluss.Enabled = true;
            Button_Storno.Enabled = true;
            Button_Gutschrift.Enabled = true;
            Button_Exit.Enabled = true;
            ZVTKey.Close();
        }

        void Button_Diagnose_Click(object sender, EventArgs e)
        {
            Funktion = 1;
            SetRegistryParameter();
            EasyZVT_Worker.RunWorkerAsync();
        }

        void Button_Exit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        void Button_Tagesabschluss_Click(object sender, EventArgs e)
        {
            Funktion = 2;
            SetRegistryParameter();
            EasyZVT_Worker.RunWorkerAsync();
        }

        void Button_Storno_Click(object sender, EventArgs e)
        {
            Funktion = 3;
            SetRegistryParameter();
            EasyZVT_Worker.RunWorkerAsync();
        }

        void Button_Gutschrift_Click(object sender, EventArgs e)
        {
            Funktion = 4;
            SetRegistryParameter();
            EasyZVT_Worker.RunWorkerAsync();
        }

        void PDocument_SeiteDrucken(object sender, PrintPageEventArgs e)
        {
            Font Textfont = TextBox_Kassenbon.Font;
            int Hoehe, Breite;
            int Links, Oben;

            var withBlock = PDocument.DefaultPageSettings;
            Hoehe = withBlock.PaperSize.Height - withBlock.Margins.Top - withBlock.Margins.Bottom;
            Breite = withBlock.PaperSize.Width - withBlock.Margins.Left - withBlock.Margins.Right;
            // Breite = 400 ' = 4 zoll = 10 cm 
            Links = PDocument.DefaultPageSettings.Margins.Left;
            Oben = PDocument.DefaultPageSettings.Margins.Top;

            // Einen Rahmen zeichnen
            Rectangle Rahmen = new Rectangle(Links, Oben, Breite, Hoehe);
            e.Graphics.DrawRectangle(Pens.Blue, Rahmen);

            // In den Rahmen den Kassenbontext zeichnen
            StringFormat StrFormat;
            StrFormat = new StringFormat(StringFormatFlags.LineLimit);
            e.Graphics.DrawString(TextBox_Kassenbon.Text, Textfont, Brushes.Black, Rahmen, StrFormat);
            e.HasMorePages = false;
        }

        void Button_Kundenbeleg_drucken_Click(object sender, EventArgs e)
        {
            PrintDialog PD = new PrintDialog();
            PD.UseEXDialog = true; // Notwendig für x64
            PDocument = new PrintDocument();
            PDocument.PrintPage += PDocument_SeiteDrucken;

            TextBox_Kassenbon.Text = "Kassenbon EasyZVT DemoApp" + Environment.NewLine;
            TextBox_Kassenbon.Text += "KasseNr: " + TextBox_KasseNr.Text + Environment.NewLine + Environment.NewLine;

            if (Betrag > 0 & Funktion == 0)
            {
                TextBox_Kassenbon.Text += "Testartikel Menge 1 Betrag " + (Convert.ToDouble(Betrag) / 100.0).ToString("###,##0.00") + Environment.NewLine;
                TextBox_Kassenbon.Text += "Gesamtbetrag               " + (Convert.ToDouble(Betrag) / 100.0).ToString("###,##0.00") + Environment.NewLine;
                TextBox_Kassenbon.Text += "Kartenzahlung              " + (Convert.ToDouble(Betrag) / 100.0).ToString("###,##0.00") + Environment.NewLine;
            }

            TextBox_Kassenbon.Text += TextBox_Drucktext.Text; // Hier den Kundenbeleg hinzufügen

            if (PD.ShowDialog() == DialogResult.OK)
            {
                PDocument.Print();
            }
        }

        void Button_Haendlerbeleg_drucken_Click(object sender, EventArgs e)
        {
            PrintDialog PD = new PrintDialog();
            PD.UseEXDialog = true; // Notwendig für x64
            PDocument = new PrintDocument();
            PDocument.PrintPage += PDocument_SeiteDrucken;

            TextBox_Kassenbon.Text = "Kassenbon EasyZVT DemoApp" + Environment.NewLine;
            TextBox_Kassenbon.Text += "KasseNr: " + TextBox_KasseNr.Text + Environment.NewLine + Environment.NewLine;

            if (Betrag > 0 & Funktion == 0)
            {
                TextBox_Kassenbon.Text += "Testartikel Menge 1 Betrag " + (Convert.ToDouble(Betrag) / 100.0).ToString("###,##0.00") + Environment.NewLine;
                TextBox_Kassenbon.Text += "Gesamtbetrag               " + (Convert.ToDouble(Betrag) / 100.0).ToString("###,##0.00") + Environment.NewLine;
                TextBox_Kassenbon.Text += "Kartenzahlung              " + (Convert.ToDouble(Betrag) / 100.0).ToString("###,##0.00") + Environment.NewLine;
            }

            TextBox_Kassenbon.Text += TextBox_Drucktext2.Text; // Hier den Händlerbeleg hinzufügen

            if (PD.ShowDialog() == DialogResult.OK)
            {
                PDocument.Print();
            }
        }
    }
}
